#include <SoftwareSerial.h>

#define rxPin 11  //connect to pin 3 Omron (tx)
#define txPin 10  //connect to pin 2 Omron (rx)
SoftwareSerial mySerial(rxPin, txPin); //rx,tx

//Constants:
const int ForceSensorPin = A0; //pin A0 to read analog input
const int Electromagnet = 3; // magnet keeps dog door closed
const int ButtonPin = 8;

//Variables:
int ForceSensorValue; //save analog value of magnet

int incomingByte = 0; //omron sensor sends info about person through bytes. 
int test = 0;

byte SetExecuteDetection[] = {0xFE, 0x04, 0x03, 0x00, 0x40, 0x01, 0x00};

byte dummyArray[1000];
int dummyCount = 0;
int dummyLenght = 0;

byte numberOfFacesDetected = 0; //looks for faces
byte numberOfHappyPeople = 0; //looks for happy faces
byte isThereABall = 0; //looks for ball presence

// de-comment following rule to turn on println not interpreted by Processing; meant for testing sensor working, printing more information
// #define TESTMODE

// comment following rule when in testmode
#define SENDMODE



void setup() {
  pinMode(Electromagnet, OUTPUT);
  pinMode(ButtonPin, INPUT);
  Serial.begin(9600);
  mySerial.begin(9600);
}

void loop() {
  checkForBallBack();

  digitalWrite(Electromagnet, HIGH);              //Turn the Electromagnet on (HIGH is the voltage level)
  ForceSensorValue = analogRead(ForceSensorPin);  //Read and save analog value from potentiometer
  if (ForceSensorValue > 100){                    //When the value is above 100 it means that there is a ball present
    isThereABall = 1;
  } else {
    isThereABall = 0;
  }
  
  functionSetExecuteDetection(); //read incoming bytes
  while (mySerial.available() > 0)
  {
    incomingByte = mySerial.read();
    dummyArray[dummyCount] = incomingByte;

#ifdef TESTMODE
    Serial.print(dummyArray[dummyCount], HEX);// TESTMODE is for testing
#endif

    dummyCount++;
    dummyLenght++;
  }
#ifdef TESTMODE
  Serial.println(" ");
  Serial.print("dummyLenght ");
  Serial.println(dummyLenght);
#endif

  numberOfFacesDetected = (dummyArray[dummyCount - dummyLenght + 8]);

#ifdef TESTMODE
  Serial.print("Number of faces detected ");
  Serial.println(numberOfFacesDetected);
#endif

  switch (numberOfFacesDetected) {
    case 0:
      dummyCount = 0;
      dummyLenght = 0;
      break;
    case 1:
      ReadGazeAndExpressionEstimationOne();
      break;
    case 2:
      ReadGazeAndExpressionEstimationTwo();
      break;
    case 3:
      ReadGazeAndExpressionEstimationThree();
      break;
  }

  dummyCount = 0;
  dummyLenght = 0;

#ifdef SENDMODE //SENDMODE is what we send to Processing through serial
  Serial.print(numberOfFacesDetected);
  Serial.print(numberOfHappyPeople);
  Serial.print(isThereABall);
#endif

}


void functionSetExecuteDetection()
{
  //Serial.println("Set Execution Detection");
  for (int x = 0; x < 7; x++)
  {
    mySerial.write(SetExecuteDetection[x]);
    //Serial.println(SetExecuteDetection[x],HEX);
  }
  int delayNormal = 1000;
  delay(delayNormal); //change the delay according to your features
}

void checkForBallBack() { //when ball is back ball will roll out of the door again.
  while (Serial.available() > 0) {
    char inChar = Serial.read();
    //Serial.println(inChar);
    if (inChar == 'b') {
      digitalWrite(Electromagnet, LOW);
      delay(1000);
    }
  }
}


void ReadGazeAndExpressionEstimationOne()//(2 bytes gaze + 6 bytes expression)
{
  char yawAngle = 0;
  char pitchAngle = 0;
  int Happiness = 0;
  char ExpressDegree = 0;

  yawAngle = dummyArray[dummyCount - 8];
  pitchAngle = dummyArray[dummyCount - 7];
  Happiness = dummyArray[dummyCount - 5];
  ExpressDegree = dummyArray[dummyCount - 1];

  // person 1 gaze
  if (yawAngle < 0 && pitchAngle < 0)
  {
#ifdef TESTMODE
    Serial.println("Person one looks down Right");
#endif
  }
  if (yawAngle > 0 && pitchAngle > 0)
  {
#ifdef TESTMODE
    Serial.println("Person one looks up Left");
#endif
  }
  if (yawAngle < 0 && pitchAngle > 0)
  {
#ifdef TESTMODE
    Serial.println("Person one looks up Right");
#endif
  }
  if (yawAngle > 0 && pitchAngle < 0)
  {
#ifdef TESTMODE
    Serial.println("Person one looks down Left");
#endif
  }

  //expression degree: Happy closer to +100 ; Surprise,Anger,Sadness closer to -100
  if (Happiness > 30 && ExpressDegree > 0) {
#ifdef TESTMODE
    Serial.println("Person One is happy.");
#endif
    numberOfHappyPeople = 1;
  }
  else {
    numberOfHappyPeople = 0;
  }
}

void ReadGazeAndExpressionEstimationTwo()//(2 bytes gaze + 6 bytes expression)
{
  char yawAnglePersonOne = 0;
  char pitchAnglePersonOne = 0;
  int HappinessPersonOne = 0;
  char ExpressDegreePersonOne = 0;
  char yawAnglePersonTwo = 0;
  char pitchAnglePersonTwo = 0;
  int HappinessPersonTwo = 0;
  char ExpressDegreePersonTwo = 0;

  yawAnglePersonTwo = dummyArray[dummyCount - 8];
  pitchAnglePersonTwo = dummyArray[dummyCount - 7];
  HappinessPersonTwo = dummyArray[dummyCount - 5];
  ExpressDegreePersonTwo = dummyArray[dummyCount - 1];
  yawAnglePersonOne = dummyArray[dummyCount - 16];
  pitchAnglePersonOne = dummyArray[dummyCount - 15];
  HappinessPersonOne = dummyArray[dummyCount - 13];
  ExpressDegreePersonOne = dummyArray[dummyCount - 9];

  // person 1 gaze
  if (yawAnglePersonOne < 0 && pitchAnglePersonOne < 0)
  {
#ifdef TESTMODE
    Serial.println("Person One looks down right");
#endif
  }
  if (yawAnglePersonOne > 0 && pitchAnglePersonOne > 0)
  {
#ifdef TESTMODE
    Serial.println("Person One looks up left");
#endif
  }
  if (yawAnglePersonOne < 0 && pitchAnglePersonOne > 0)
  {
#ifdef TESTMODE
    Serial.println("Person One looks up rightt");
#endif
  }
  if (yawAnglePersonOne > 0 && pitchAnglePersonOne < 0)
  {
#ifdef TESTMODE
    Serial.println("Person One looks down left");
#endif
  }

  //person 2 gaze
  if (yawAnglePersonTwo < 0 && pitchAnglePersonTwo < 0)
  {
#ifdef TESTMODE
    Serial.println("Person Two looks down right");
#endif
  }
  if (yawAnglePersonTwo > 0 && pitchAnglePersonTwo > 0)
  {
#ifdef TESTMODE
    Serial.println("Person Two looks up left");
#endif
  }
  if (yawAnglePersonTwo < 0 && pitchAnglePersonTwo > 0)
  {
#ifdef TESTMODE
    Serial.println("Person Two looks up rightt");
#endif
  }
  if (yawAnglePersonTwo > 0 && pitchAnglePersonTwo < 0)
  {
#ifdef TESTMODE
    Serial.println("Person Two looks down left");
#endif
  }

  //expression degree: Happy closer to +100 ; Surprise,Anger,Sadness closer to -100

  if (HappinessPersonTwo > 30 && ExpressDegreePersonTwo > 0 && HappinessPersonOne > 30 && ExpressDegreePersonOne > 0) {
#ifdef TESTMODE
    Serial.println("Person One and Two are happy");
#endif
    numberOfHappyPeople = 2;
  }
  else if (HappinessPersonOne > 30 && ExpressDegreePersonOne > 0) {
#ifdef TESTMODE
    Serial.println("Person One is happy");
#endif
    numberOfHappyPeople = 1;
  }
  else if (HappinessPersonTwo > 30 && ExpressDegreePersonTwo > 0) {
#ifdef TESTMODE
    Serial.println("Person Two is happy");
#endif
    numberOfHappyPeople = 1;
  }
  else {
    numberOfHappyPeople = 0;
  }
}

void ReadGazeAndExpressionEstimationThree()//(2 bytes gaze + 6 bytes expression)
{
  char yawAnglePersonOne = 0;
  char pitchAnglePersonOne = 0;
  int HappinessPersonOne = 0;
  char ExpressDegreePersonOne = 0;
  char yawAnglePersonTwo = 0;
  char pitchAnglePersonTwo = 0;
  int HappinessPersonTwo = 0;
  char ExpressDegreePersonTwo = 0;
  char yawAnglePersonThree = 0;
  char pitchAnglePersonThree = 0;
  int HappinessPersonThree = 0;
  char ExpressDegreePersonThree = 0;

  yawAnglePersonThree = dummyArray[dummyCount - 8];
  pitchAnglePersonThree = dummyArray[dummyCount - 7];
  HappinessPersonThree = dummyArray[dummyCount - 5];
  ExpressDegreePersonThree = dummyArray[dummyCount - 1];
  yawAnglePersonTwo = dummyArray[dummyCount - 16];
  pitchAnglePersonTwo = dummyArray[dummyCount - 15];
  HappinessPersonTwo = dummyArray[dummyCount - 13];
  ExpressDegreePersonTwo = dummyArray[dummyCount - 9];
  yawAnglePersonOne = dummyArray[dummyCount - 24];
  pitchAnglePersonOne = dummyArray[dummyCount - 23];
  HappinessPersonOne = dummyArray[dummyCount - 21];
  ExpressDegreePersonOne = dummyArray[dummyCount - 17];

  // person 1 gaze
  if (yawAnglePersonOne < 0 && pitchAnglePersonOne < 0)
  {
#ifdef TESTMODE
    Serial.println("Person One looks down right");
#endif
  }
  if (yawAnglePersonOne > 0 && pitchAnglePersonOne > 0)
  {
#ifdef TESTMODE
    Serial.println("Person One looks up left");
#endif
  }
  if (yawAnglePersonOne < 0 && pitchAnglePersonOne > 0)
  {
#ifdef TESTMODE
    Serial.println("Person One looks up rightt");
#endif
  }
  if (yawAnglePersonOne > 0 && pitchAnglePersonOne < 0)
  {
#ifdef TESTMODE
    Serial.println("Person One looks down left");
#endif
  }

  //person 2 gaze
  if (yawAnglePersonTwo < 0 && pitchAnglePersonTwo < 0)
  {
#ifdef TESTMODE
    Serial.println("Person Two looks down right");
#endif
  }
  if (yawAnglePersonTwo > 0 && pitchAnglePersonTwo > 0)
  {
#ifdef TESTMODE
    Serial.println("Person Two looks up left");
#endif
  }
  if (yawAnglePersonTwo < 0 && pitchAnglePersonTwo > 0)
  {
#ifdef TESTMODE
    Serial.println("Person Two looks up rightt");
#endif
  }
  if (yawAnglePersonTwo > 0 && pitchAnglePersonTwo < 0)
  {
#ifdef TESTMODE
    Serial.println("Person Two looks down left");
#endif
  }

  //person 3 gaze
  if (yawAnglePersonThree < 0 && pitchAnglePersonThree < 0)
  {
#ifdef TESTMODE
    Serial.println("Person Three looks down right");
#endif
  }
  if (yawAnglePersonThree > 0 && pitchAnglePersonThree > 0)
  {
#ifdef TESTMODE
    Serial.println("Person Three looks up left");
#endif
  }
  if (yawAnglePersonThree < 0 && pitchAnglePersonThree > 0)
  {
#ifdef TESTMODE
    Serial.println("Person Three looks up rightt");
#endif
  }
  if (yawAnglePersonThree > 0 && pitchAnglePersonThree < 0)
  {
#ifdef TESTMODE
    Serial.println("Person Three looks down left");
#endif
  }


  //expression degree: Happy closer to +100 ; Surprise,Anger,Sadness closer to -100
  if (HappinessPersonThree > 30 && ExpressDegreePersonThree > 0 && HappinessPersonTwo > 30 && ExpressDegreePersonTwo > 0 && HappinessPersonOne > 30 && ExpressDegreePersonOne > 0) {
#ifdef TESTMODE
    Serial.println("Person One, Two and Three are happy");
#endif
    numberOfHappyPeople = 3;
  }
  else if ((HappinessPersonTwo > 30 && ExpressDegreePersonTwo > 0 && HappinessPersonOne > 30 && ExpressDegreePersonOne > 0) || (HappinessPersonTwo > 30 && ExpressDegreePersonTwo > 0 && HappinessPersonThree > 30 && ExpressDegreePersonThree > 0) || (HappinessPersonOne > 30 && ExpressDegreePersonOne > 0 && HappinessPersonThree > 30 && ExpressDegreePersonThree > 0)) {
#ifdef TESTMODE
    Serial.println("Person One and Two are happy");
#endif
    numberOfHappyPeople = 2;
  }
  else if (HappinessPersonOne > 30 && ExpressDegreePersonOne > 0) {
#ifdef TESTMODE
    Serial.println("Person One is happy");
#endif
    numberOfHappyPeople = 1;
  }
  else if (HappinessPersonTwo > 30 && ExpressDegreePersonTwo > 0) {
#ifdef TESTMODE
    Serial.println("Person Two is happy");
#endif
    numberOfHappyPeople = 1;
  }
  else if (HappinessPersonThree > 30 && ExpressDegreePersonThree > 0) {
#ifdef TESTMODE
    Serial.println("Person Three is happy");
#endif
    numberOfHappyPeople = 1;
  }
  else {
    numberOfHappyPeople = 0;
  }
}


