import processing.video.*;
import processing.serial.*; 
import nl.tue.id.oocsi.*; 

OOCSI oocsi;
Serial myPort;

String readSerialData = "000";

// 6 categories (0-5) the last category is the 'throwing the ball category'
String[][] movieNamesArray = { 
  { 
    "Naamloos18.avi", "Naamloos1.avi", "Naamloos6.avi"
  }
  , 
  { 
    "Naamloos20.avi", "Naamloos2.avi", "Naamloos38.avi"
  }
  , 
  { 
    "Naamloos3.avi", "Naamloos15.avi", "Naamloos19.avi"
  }
  , 
  { 
    "Naamloos4.avi", "Naamloos8.avi", "Naamloos21.avi"
  }
  , 
  { 
    "Naamloos22.avi", "Naamloos9.avi", "Naamloos31.avi"
  }
  , 
  { 
    "Naamloos38.avi", "Naamloos12.avi", "Naamloos23.avi"
  }
  , 
  { 
    "Naamloos14_1.avi", "Naamloos24.avi", "Naamloos26_1.avi", "Naamloos29_1.avi"
  }
};


Movie[][] myMovie;
int movieCategory;
int movieIndex;

// data csv file structure
Data myData;

int personID = 0; 
int personWatchingID = 0;

// comming in from serial
char numberOfAttentivePeople = readSerialData.charAt(0);
char numberOfHappyPeople = readSerialData.charAt(1);
char isThereABall = readSerialData.charAt(2);


int ball = 0; 
int lastBall = 0;

int attentivePeople = 0;
int happyPeople = 0;

// to check for change
int numberOfPeopleThatHadAttention = 0; 

// check for attention / happiness and eventually movie appeal
int onePersonHasAttention = 0;
int onePersonHappinessMovie = 0;
int morePeopleHaveAttention = 0;
int morePeopleHappinessMovie = 0;
float movieAppealValueOnePerson;
float finalMovieAppealValueOnePerson = 0;
float morePeopleMovieAppealValue = 0;
int videoAttractsPeople = 0;

// to make sure categories don't dissapear those values are 1
float attractPeopleValueToSave = 1;
float valueToSave = 1;
float morePeopleValueToSave = 1;

// used when saving data to pick next category
float total = 0;
int savedDataSet;
int rnd;
int totalChanceCategories = 0;
int pickedCategory;

void setup() {
  size(640, 360, P2D);
  background(0);

  // initialize communication
  oocsi = new OOCSI(this, "prototype", "oocsi.id.tue.nl"); 
  myPort = new Serial(this, "COM21", 9600);

  // used to read out data file .csv
  int amountOfDatasets = 5; // 1: attractAttention, 2: dataTrudie, 3: dataBert, 4: dataGerda, 5:morePeople
  int amountOfMovieCategories = 6; //elke row is 1 filmcategorie. Er wordt dus bijgehouden per state wat de appeal is van de filmcategorie.
  myData = new Data(amountOfDatasets, amountOfMovieCategories, "datav3.csv");

  // initial category and video pick
  movieCategory = 1; // Set the default category
  movieIndex = int(random(movieNamesArray[movieCategory].length)); // Pick one from the current category

  // Calculate how many videos are in the categories, which are in the multi-array
  int maxRowLength = 0;
  for (int i = 0; i < movieNamesArray.length; i++) {
    for (int j = 0; j < movieNamesArray[i].length; j++) {
      if (maxRowLength < movieNamesArray[i].length) {
        maxRowLength = movieNamesArray[i].length;
      }
    }
  }
  // Define the length of the multi array object
  myMovie = new Movie[movieNamesArray.length][maxRowLength];

  for (int i = 0; i < movieNamesArray.length; i++) {
    for (int j = 0; j < movieNamesArray[i].length; j++) {
      myMovie[i][j] = new Movie(this, movieNamesArray[i][j]);
    }
  }
  myMovie[movieCategory][movieIndex].play(); // start playing
}

// play the movie
void movieEvent(Movie m) {
  m.read();
}

void draw() {
  // draw movie frame
  image(myMovie[movieCategory][movieIndex], -10, 0, width+20, height);

  numberOfPeopleThatHadAttention = attentivePeople; 

  //get Sensor Data and interpret
  while (myPort.available () > 2) {
    String val = myPort.readString();
    readSerialData = val;
  } 

  numberOfAttentivePeople = readSerialData.charAt(0);
  numberOfHappyPeople = readSerialData.charAt(1);
  isThereABall = readSerialData.charAt(2);

  // quick fix ASCII
  attentivePeople = int(numberOfAttentivePeople)-48;
  happyPeople = int(numberOfHappyPeople)-48;
  ball = int(isThereABall)-48;

  // check how many people have attention and start counting attention / happiness based on that.
  // if one person is watching, ask via OOCSI who?
  checkHowManyPeopleHaveAttention();
  askResearcherWhoWatches();

  // check for the ball if it is thrown into the prototype
  if (movieCategory == 5) {
    checkForBallBackEvent();
  } else {
    checkForBallEvent();
  }

  // at the end of a movie determine and save the movie appeal
  if (myMovie[movieCategory][movieIndex].time() >= (myMovie[movieCategory][movieIndex].duration() - frameFix)) {
    println("finished");
    finalMovieAppealValueOnePerson = ((onePersonHasAttention/100)*(1+onePersonHappinessMovie/100))/myMovie[movieCategory][movieIndex].duration();
    println(finalMovieAppealValueOnePerson);

    // save the data to the correct cattegory / person
    saveTheData();

    // reset values
    onePersonHasAttention = 0;
    onePersonHappinessMovie = 0;

    // if nobody is watching anymore at the end of the movie prepare to ask again 'to ask who is watching' if someone starts watching
    if (attentivePeople == 0) {
      personID = 0;
    }

    personWatchingID = 0;

    // use values from the .csv to determine semi-randomly from which category the next video should be picked and pick new movie (semi-randomly is based on the learning algorithm)
    determineCurrentCategory();
  }

  lastBall = ball;
}

void saveTheData() {

  // determine movie appeal
  finalMovieAppealValueOnePerson = ((onePersonHasAttention/100)*(1+onePersonHappinessMovie/100))/myMovie[movieCategory][movieIndex].duration();

  // save the data to the correct cattegory / person
  //saving personal data
  switch (personWatchingID) {

  case 5:
    // add movieAppeal to Trudie's data
    valueToSave = myData.get(movieCategory, 1) + finalMovieAppealValueOnePerson;
    myData.save (movieCategory, 1, valueToSave);

    println ("data saved to Trudie");
    println (valueToSave);
    break;

  case 6:
    // add movieAppeal to Bert's data
    valueToSave = myData.get(movieCategory, 2) + finalMovieAppealValueOnePerson;
    myData.save (movieCategory, 2, valueToSave);

    println ("data saved to Bert");
    println (valueToSave);
    break;

  case 7:
    // add movieAppeal to Gerda's data
    valueToSave = myData.get(movieCategory, 3) + finalMovieAppealValueOnePerson;
    myData.save (movieCategory, 3, valueToSave);

    println ("data saved to Gerda");
    println (valueToSave);
    break;
  }

  //saving attraction data or more people data
  // videoAttractsPeople value --> save if a person started watching the video --> if a person started watching the video increase likelihood for category in algorithm
  attractPeopleValueToSave = myData.get(movieCategory, 0) + videoAttractsPeople;
  myData.save (movieCategory, 0, attractPeopleValueToSave);
  println ("data saved to attraction");
  println (attractPeopleValueToSave);

  // morePeopleMovieAppeal --> save when more people are watching the videos which videos attract more people
  morePeopleValueToSave = myData.get(movieCategory, 0) + morePeopleMovieAppealValue;
  myData.save (movieCategory, 4, morePeopleValueToSave);

  myData.saveFile("data/datav3.csv");
}

// check what the sensor reports
void checkHowManyPeopleHaveAttention() {

  // check for change from no one to someone --> if yes --> video attracts people
  if (attentivePeople > 0 && numberOfPeopleThatHadAttention == 0 && personWatchingID == 0)
  {
    videoAttractsPeople ++;
  }

  // when the sensor reports that someone is watching, determine how many and proceed from there.
  switch (attentivePeople) {

  case 1:
    // check who is watching
    personWatchingID = personID;
    println("personWatching");
    println(personWatchingID);
    // Start counting how long the attention is kept by this person.
    onePersonHasAttention ++;

    // Start counting how many times the person is happy.
    if (happyPeople == 1) {
      onePersonHappinessMovie ++;
    }
    break;

  case 2:
    // Start counting how long the attention is kept by more than one person.
    morePeopleHaveAttention ++;
    // Start counting how many times the people are happy.
    if (happyPeople > 0) {
      morePeopleHappinessMovie =  morePeopleHappinessMovie + happyPeople;
    } 
    break;

  case 3:
    // same as case 2.
    // Start counting how long the attention is kept by more than one person.
    morePeopleHaveAttention ++;
    // Start counting how many times the people are happy.
    if (happyPeople > 0) {
      morePeopleHappinessMovie = morePeopleHappinessMovie + happyPeople;
    } 
    break;
  }
}

void determineCurrentCategory() {

  // determine where data should be saved 
  if (attentivePeople == 0) {
    // attract people
    savedDataSet = 0;
  } else if (attentivePeople == 1 && personWatchingID == 5) {
    // Trudie
    savedDataSet = 1;
  } else if (attentivePeople == 1 && personWatchingID == 6) {
    // Bert
    savedDataSet = 2;
  } else if (attentivePeople == 1 && personWatchingID == 7) {
    // Gerda
    savedDataSet = 3;
  } else if (attentivePeople > 1) {
    // keep attention of more people
    savedDataSet = 4;
  } else if (attentivePeople == 1 && personWatchingID == 0) {
    // continue without saving information
  }

// total value of all category likelihoods
  total = myData.get(0, savedDataSet)+ myData.get(1, savedDataSet)+ myData.get(2, savedDataSet)+ myData.get(3, savedDataSet)+ myData.get(4, savedDataSet) + myData.get(5, savedDataSet);

// chance for each category
  int chanceCategoryOne = (int)((myData.get(0, 0) / total)*100);
  int chanceCategoryTwo = (int)((myData.get(1, 0) / total)*100);
  int chanceCategoryThree = (int)((myData.get(2, 0) / total)*100);
  int chanceCategoryFour = (int)((myData.get(3, 0) / total)*100);
  int chanceCategoryFive = (int)((myData.get(4, 0) / total)*100);
  int chanceCategorySix = (int)((myData.get(5, 0) / total)*100);
  totalChanceCategories = (chanceCategoryOne + chanceCategoryTwo + chanceCategoryThree + chanceCategoryFour + chanceCategoryFive + chanceCategorySix);

// array filled with category numbers based on their chance
  int[] randomArray = new int[totalChanceCategories];

  //  println("total");
  //  println(total);
  //  
  //  println("chance");
  //  println(chanceCategoryOne);
  //  println(chanceCategoryTwo);
  //  println(chanceCategoryThree);
  //  println(chanceCategoryFour);
  //  println(chanceCategoryFive);

// fill array with category numbers based on their chance
  for (int i = 0; i<chanceCategoryOne; i++) {
    randomArray [i] = 0;
  }
  for (int i = chanceCategoryOne; i< (chanceCategoryOne + chanceCategoryTwo); i++) {
    randomArray [i] = 1;
  }
  for (int i = (chanceCategoryOne + chanceCategoryTwo); i<(chanceCategoryOne + chanceCategoryTwo + chanceCategoryThree); i++) {
    randomArray [i] = 2;
  }
  for (int i = (chanceCategoryOne + chanceCategoryTwo + chanceCategoryThree); i< (chanceCategoryOne + chanceCategoryTwo + chanceCategoryThree + chanceCategoryFour); i++) {
    randomArray [i] = 3;
  }
  for (int i = (chanceCategoryOne + chanceCategoryTwo + chanceCategoryThree + chanceCategoryFour); i < (chanceCategoryOne + chanceCategoryTwo + chanceCategoryThree + chanceCategoryFour + chanceCategoryFive); i++) {
    randomArray [i] = 4;
  }
  for (int i = (chanceCategoryOne + chanceCategoryTwo + chanceCategoryThree + chanceCategoryFour + chanceCategoryFive); i < (totalChanceCategories); i++) {
    randomArray [i] = 5;
  }

 // choose random number ranging between 0 and maximum number in the array
  rnd = (int) random(totalChanceCategories);

// choose the category number at the location in the array which is picked by the random, chances are higher for the categories that had higher appeal values as they appear more often in the array
  pickedCategory = randomArray [rnd];

// set movieCategory to picked category and select next movie to play
  changeCategory(pickedCategory);
  println(pickedCategory);

}


// picks a random movie within the category which is currently supposed to be playing
void pickRandomVideoFromCurrentCategory() {
  if (movieNamesArray[movieCategory].length <= 1)  return;
  int rndom; // keep picking a new index till got a diff. 1:
  rnd = (int) random(movieNamesArray[movieCategory].length);
  // assign newly picked random value to index:
  myMovie[movieCategory][movieIndex = rnd].stop(); // stops the video.
  myMovie[movieCategory][movieIndex].play(); // and start playing it.
}

// change category of movies
void changeCategory(int category) {
  movieCategory = category;
  pickRandomVideoFromCurrentCategory();
}

// look for OOCSI data = data about the person that is watching
void handleOOCSIEvent(OOCSIEvent event) {
  personID = event.getInt("personWatching", 0);
  println ("personID oocsi");
  println (personID);
}

// ask for who is watching --> send oocsi data --> happens when 1 person is watching
void askResearcherWhoWatches() {
  if (attentivePeople == 1 && numberOfPeopleThatHadAttention != 1) {
    println("We see one person, we ask who it is!");
    oocsi.channel("researcher").data("WhoIsWatchingQuestion", attentivePeople).send();
  } else if (attentivePeople == 0 && numberOfPeopleThatHadAttention != 0) {
    println("Nobody watching anymore, stop asking...");
    oocsi.channel("researcher").data("WhoIsWatchingQuestion", attentivePeople).send();
  }
}

// checks if the ball is thrown (in the prototype) or not.
void checkForBallEvent() {
  if (ball == 1 && lastBall != 1) {
    changeCategory(5); // Pick random video from category #3.
  }
}

//check if the ball video ended and the dog brought the ball back, based on that the ball is set free from the protoype
void checkForBallBackEvent() {
  if (myMovie[movieCategory][movieIndex].time() >= (myMovie[movieCategory][movieIndex].duration() - frameFix)) {
    println("Ball should go back!");
    sendBallBackCommand();
    changeCategory(pickedCategory);
  }
}

//sends to arduino to release the ball
void sendBallBackCommand() {
  myPort.write('b');
}

// manual testing
// press 'q' before closing, otherwise the serial might not close properly and the program cannot run anymore the next time
void keyPressed() {
  if (key == 'q') {
    myData.saveFile("data/datav3.csv");
    myPort.stop();
  } else if (key == 'm') {
    myMovie[movieCategory][movieIndex].pause();
  } else if (key == 'p') {
    pickRandomVideoFromCurrentCategory();
  } else if (key == 'o') {
    myMovie[movieCategory][movieIndex].play();
  } else if (key == 'a') {
    changeCategory(0);
  } else if (key == 's') {
    changeCategory(1);
  } else if (key == 'd') {
    changeCategory(2);
  } else if (key == 'l') {
    sendBallBackCommand();
  }
}

